@@ -74,7 +74,9 @@ CLR_FEST = {
     False: clr.LMODEL_V_PMODEL_S2[-1]
 }
 CLR_STAGE = {
+    'Brinewater Springs': clr.GRAPE_V_TURQUOISE_S2[0], 
     'Eeltail Alley': clr.PINK_V_ORANGE_S1[1],
+    'Flounder Heights': clr.SWEATER_V_SOCKS_S2[0],
     'Hagglefish Market': clr.LPINK_V_BLUE_S2[1],
     'Hammerhead Bridge': clr.MAYO_V_KETCHUP_S2[1],
     'Inkblot Art Academy': clr.LPINK_V_BLUE_S2[0],
