@@ -2,10 +2,14 @@
 import squarify
 import numpy as np
 import seaborn as sns
+from pywaffle import Waffle
 import matplotlib.pyplot as plt
+from math import radians, log10
 from matplotlib.patches import Rectangle
 import SplatStats.constants as cst
 import SplatStats.stats as stats
+import SplatStats.auxiliary as aux
+import SplatStats.colors as clr
 import SplatStats.plotsAux as paux
 import matplotlib
 matplotlib.rcParams['font.family'] = ['monospace']
@@ -592,7 +596,7 @@ def plotMatchTypeBars(
     """Generates a grid of seaborn plots with stats broken down by match type and stage.
 
     Args:
-        stagesByTypeFlat (dataframe): _description_
+        stagesByTypeFlat (dataframe): Dataframe obtained by running ammendStagesStatsByType on the calcStagesStatsByType structure.
         metric (str): Main metric for bar sizes.
         aggMetrics (tuple of strings): These two metrics are used to calculate a fraction of metric1/metric2 on the aggregate value displayed. Defaults to ('win', 'total matches').
         yRange (tuple, optional): y-axis plot range. Defaults to (0, 1).
@@ -709,17 +713,178 @@ def plotRanking(
         normalized=True, yLim=None, xLim=None,
         colors=['#C70864', '#C920B7', '#4B25C9', '#830B9C', '#2CB721'],
         categories=['kill', 'death', 'assist', 'paint', 'special'],
-        widthBar=0.9,
+        widthBar=0.9, title=True, pad=0, titlePos=(0.975, .9),
         **kwargs
     ):
+    """Generates barcharts on stats for player's ranking across categories.
+
+    Args:
+        figAx (tuple): (fig, ax) tuple as initialized by matplotlib (plt.subplots).
+        rankingDF (dataframe): Either getPlayerAlliedRanking or getPlayerFullRanking dataframe on battle data.
+        normalized (bool, optional): If true, data is scaled to proportions (0 to 1) instead of raw frequencies. Defaults to True.
+        xLim (tuple, optional): Plot limits on x axis Defaults to None.
+        yLim (tuple, optional): Plot limits on y axis. Defaults to None.
+        colors (list, optional): Color palette for bars. Defaults to ['#C70864', '#C920B7', '#4B25C9', '#830B9C', '#2CB721'].
+        categories (list, optional): Categories for the ranks (in order). Defaults to ['kill', 'death', 'assist', 'paint', 'special'].
+        widthBar (float, optional): Bars width. Defaults to 0.9.
+        title (bool, optional): If true, the title of the plot is printed. Defaults to True.
+        titlePos (tuple, optional): Position for plots' title. Defaults to (0.975, .9).
+        pad (int, optional): Padding for plots along the grid. Defaults to 0.
+
+    Returns:
+        (fix, ax): Matplotlib's fig and ax objects.
+    """    
     (fig, axes) = figAx
+    fig.tight_layout(pad=pad)
     for (col, ax, cat) in zip(colors, axes, categories):
         rankingDF[cat].value_counts(normalize=normalized).sort_index().plot(
             ax=ax, kind='bar', color=col, width=widthBar,
-            title=cat, **kwargs
+            **kwargs
         )
+        if title:
+            ax.text(
+                titlePos[0], titlePos[1], cat, 
+                transform=ax.transAxes, ha="right"
+            )
         if yLim:
             ax.set_ylim(*yLim)
         if xLim:
             ax.set_xlim(*xLim)
-    return (fig, axes)
\ No newline at end of file
+    return (fig, axes)
+
+
+def plotWaffleStat(
+        figAx, playerHistory,
+        function=sum, grouping='main weapon', stat='kill',
+        rows=50, columns=50, startingLocation='NW', blockArranging='snake',
+        intervalRatioX=0.5, intervalRatioY=0.5,
+        colors=clr.ALL_COLORS, alpha=.6,
+        fmt="{:.2f}",
+        title=True,
+        legendDict={
+            'loc': 'upper left',
+            'bbox_to_anchor': (1, 1),
+            'ncol': 1,
+            'framealpha': 0,
+            'fontsize': 10
+        },
+        **kwargs
+    ):
+    """Generates a waffle plot of aggregated statistics on player's history (eg. sums of kills broken by main weapon).
+
+    Args:
+        figAx (tuple): (fig, ax) tuple as initialized by matplotlib (plt.subplots).
+        playerHistory (dataframe): Player history dataframe.
+        function (function, optional): Aggregation function for groupby. Defaults to sum.
+        grouping (str, optional): Category over which the aggregation will be done. Defaults to 'main weapon'.
+        stat (str, optional): Statistic to use in the plot. Defaults to 'kill'.
+        rows (int, optional): Number of rows for waffle. Defaults to 50.
+        columns (int, optional): Number of columns for waffle. Defaults to 50.
+        startingLocation (str, optional): Waffle's starting locations (see pywaffle's docs). Defaults to 'NW'.
+        blockArranging (str, optional): Block arranging (see pywaffle's docs). Defaults to 'snake'.
+        intervalRatioX (float, optional): Spacing over x axis. Defaults to 0.5.
+        intervalRatioY (float, optional): Spacing over y axis. Defaults to 0.5.
+        colors (list, optional): List of hex colors. Defaults to clr.ALL_COLORS.
+        alpha (float, optional): Alpha for the colors. Defaults to .6.
+        fmt (str, optional): Formating for the title number. Defaults to "{:.2f}".
+        title (bool, optional): Prints title if true. Defaults to True.
+        legendDict (dict, optional): Dictionary for labels (see pywaffle's docs). Defaults to { 'loc': 'upper left', 'bbox_to_anchor': (1, 1), 'ncol': 1, 'framealpha': 0, 'fontsize': 10 }.
+
+    Returns:
+        (fix, ax): Matplotlib's fig and ax objects.
+    """    
+    (fig, ax) = figAx
+    # Aggregate ---------------------------------------------------------------
+    df = playerHistory.groupby(grouping).agg(function)
+    cols = [
+        i+aux.alphaToHex(alpha) for i in colors[:len(df.index)]
+    ]
+    # Waffle ------------------------------------------------------------------
+    Waffle.make_waffle(
+        ax=ax,
+        rows=rows, 
+        columns=columns,
+        values=df[stat],
+        starting_location=startingLocation,
+        vertical=True,
+        block_arranging_style=blockArranging,
+        colors=cols,
+        interval_ratio_x=intervalRatioX,
+        interval_ratio_y=intervalRatioY,
+        labels=[
+            f"{k} ({float(v/sum(df[stat])*100):.2f}%)" 
+            for (k, v) in zip(df.index, df[stat])
+        ],
+        legend=legendDict,
+        **kwargs
+    )
+    if title:
+        plt.title(f"{stat} ("+fmt.format(function(df[stat]))+")")
+    return (fig, ax)
+
+
+def plotCircularBarchartStat(
+        figAx,
+        playerHistory, cat='main weapon', stat='kill', aggFun=np.sum,
+        autoRange=True, logScale=True, gStep=30,
+        rRange=(0, 270), xRange=(0, 10),
+        colors=clr.ALL_COLORS
+    ):
+    """Generates a radial barchart for 
+
+    Args:
+        figAx (tuple): (fig, ax) tuple as initialized by matplotlib (plt.subplots).
+        playerHistory (dataframe): Player history dataframe.
+        cat (str): Category over which the aggregation will be done. Defaults to 'main weapon'.
+        stat (str): Statistic to use in the plot. Defaults to 'kill'.
+        aggFun (function, optional): Aggregation function for groupby. Defaults to np.sum.
+        autoRange (bool, optional): Auto-scales the radial plot with data min and data max. Defaults to True.
+        logScale (bool, optional): Uses log-scale for the radial bars. Defaults to True.
+        gStep (int, optional): Number of equally-spaced steps for radial ticks. Defaults to 30.
+        rRange (tuple, optional): Radial angles for the plot. Defaults to (0, 270).
+        xRange (tuple, optional): Values to map onto the rRange values. Defaults to (0, 10).
+        colors (_type_, optional): List of hex colors for bars. Defaults to clr.ALL_COLORS.
+
+    Returns:
+        (fix, ax): Matplotlib's fig and ax objects.
+    """    
+    (fig, ax) = figAx
+    # Gather data -------------------------------------------------------------
+    df = playerHistory.groupby(cat).agg(aggFun)
+    df.sort_values(by=[stat], inplace=True)
+    catVals = list(df[stat])
+    # Scaling stats values ----------------------------------------------------
+    mxVal = max(catVals) if autoRange else xRange[1]
+    gGrid = np.arange(xRange[0], mxVal+mxVal/(0.5*gStep), mxVal/gStep)
+    if logScale:
+        vVals = [log10(i+1) for i in catVals]
+        xRan = (0, max(vVals) if autoRange else log10(1+xRange[1]))
+        gVals = [log10(i+1) for i in gGrid]
+    else:
+        vVals = catVals
+        xRan = (0, max(vVals) if autoRange else xRange[1])
+        gVals = gGrid
+    weapons = list(df.index)
+    angles = [np.interp(i, xRan, rRange) for i in vVals]
+    grids = [np.interp(i, xRan, rRange) for i in gVals]
+    # Generate Plot -----------------------------------------------------------
+    for (i, ang) in enumerate(angles):
+        ax.barh(i, radians(ang), color=colors[i])
+    # Gridlines and axes ------------------------------------------------------
+    ax.vlines(
+        [radians(i) for i in grids[:-1]], len(weapons)-.5, len(weapons)-.25,  
+        lw=1, colors='k', alpha=.5
+    )
+    ax.xaxis.grid(False)
+    ax.yaxis.grid(False)
+    ax.spines['polar'].set_visible(False)
+    ax.set_theta_zero_location('N')
+    ax.set_theta_direction(1)
+    ax.set_rlabel_position(0)
+    ax.set_thetagrids(rRange, [], color='#00000000')
+    ax.set_rgrids(
+        [i for i in range(len(weapons))], 
+        labels=[f' {w} ({v:.2f})' for (w, v) in zip(weapons, catVals)],
+        va='center'
+    )
+    return (fig, ax)
\ No newline at end of file
